@@ -3,20 +3,34 @@
 # See the file 'COPYING' for copying permission.
 
 import Tkinter
+import time
+import tkFont
 
 from .constants import map_color
 
 
 class Canvas(object):
-    Fps = 100
+    Fps = 60
     IntervalMs = 1000 // Fps
+    MinRefreshMs = 1
+
+    def _next_refresh(self):
+        ms = int(round(time.time() * 1000))
+        elapsed = ms - self._time
+        self._time = ms
+
+        if Canvas.IntervalMs < elapsed:
+            return Canvas.MinRefreshMs
+
+        return max(Canvas.IntervalMs - elapsed, Canvas.MinRefreshMs)
 
     def __init__(self, master, width, height):
-        self._canvas = Tkinter.Canvas(master, width=width, height=height,
-                                      background='black')
-        self._canvas.pack(fill=Tkinter.X)
+        self._canvas = Tkinter.Canvas(master, width=width, height=height, bd=2,
+                                      bg='black', highlightthickness=0)
+        self._canvas.pack(fill=Tkinter.BOTH, expand=True)
 
         self._draw_handler_fn = None
+        self._time = int(round(time.time() * 1000))
         self._draw_handler(master)
 
     def _get_widget(self):
@@ -30,7 +44,7 @@ class Canvas(object):
         if self._draw_handler_fn is not None:
             self._canvas.update_idletasks()
 
-        master.after(Canvas.IntervalMs, self._draw_handler, master)
+        master.after(self._next_refresh(), self._draw_handler, master)
 
     def destroy(self):
         self._draw_handler_fn = None
@@ -38,6 +52,9 @@ class Canvas(object):
     def set_background(self, color):
         self._canvas.config(background=map_color(color))
 
+    def get_textwidth(self, text, size, face):
+        return tkFont.Font(size=size, family=face).measure(text)
+
     def set_draw_handler(self, draw_handler):
         self._draw_handler_fn = draw_handler
 
@@ -56,7 +73,7 @@ class Canvas(object):
                            map_color(line_color))
 
     def draw_polygon(self, point_list, line_width, line_color,
-                     fill_color=None):
+                     fill_color=''):
         points = [y for x in point_list for y in x]
         self._canvas.create_polygon(points, width=line_width,
                                     fill=map_color(fill_color),
